/* ' Copyright Sibelius Software Ltd 2004 All Rights Reserved
' Do not redistribute. Obtain only from www.sibelius.com
' 
' Simple script to fix Sibelius or G7 file associations for XP Service Pack 2
' Required for Sibelius v2 or v3 or G7
' Not required for Sibelius v1
'
' If both G7 and Sibelius are installed this script will only fix the one which is currently
' associated with .sib files
'
' Once this script has been run, Sibelius or G7 will no longer be able to reset the file associations
' until this script has been undone
'
' Replace the old open verb with new open_xp_sp2 verb and make the new verb the default
' This is required since Sibelius overwrites the open verb every time it runs
' 
' A similar issue affects printing from Windows Explorer
' It is possible to add a new verb 'print_xp_sp2' to the Sibelius.Score type and Applications key.
' However this cannot be made the default printing action (e.g. when dragging a file to a printer icon)
' so we don't set it here.
'
' ============================================================
' Manual instructions for advanced users or users who can't use this script
' (e.g. if Windows Scripting Host has been disabled):
' ============================================================
'
' Start a Command Prompt (Start/Programs/Accessories).
' Execute the following commands in the command prompt
' reg copy HKCR\Sibelius.Score\shell\open HKCR\Sibelius.Score\shell\open_xp_sp2 /s /f
' reg add HKCR\Sibelius.Score\shell /ve /d "open_xp_sp2" /f
' regedit
' In the Regedit window choose Edit/Find...
' Type open_xp_sp2
' Press OK
' Navigate down to \open_xp_sp2\shell\command a few lines below
' Edit the 'Default Value' here by double clicking
' Remove the single space character between Sibelius.exe" and "%1"
' Press OK.
' Close Regedit and the command window

' ============================================================
' Start of script
' ============================================================
*/

var SibeliusProgramName; 
SibeliusProgramName = "Sibelius.exe";		// program name to search for in existing open command
var G7ProgramName; 
G7ProgramName = "G7.exe";				    // alternative program name to search for in existing open command
var RegKey;
RegKey = "HKCR\\Sibelius.Score\\shell\\";	// ProgId in registry to change
var OldVerb;
OldVerb = "open";							// The existing shell verb we're copying
// The new default shell verb we're creating.
// We need to create a new verb as our program will re-default the old verb on each run
var NewVerb;
NewVerb = "open_xp_sp2";					// The name of the new default verb that we create
var CommandStr;
CommandStr = "\\command\\";					// The key between verb and the actual value
var OldDefaultKey;
OldDefaultKey = "pre_xp_sp2_default_value";	// The key to store the previous default verb for the undo script

// The application key. This is needed in case the user has or will associate our program with other file types in explorer
var ApplicationKey;
ApplicationKey = "HKCR\\Applications\\";

var ScriptName =  "Sibelius XP SP2 Fix";

var WSHShell = new ActiveXObject("WScript.Shell");

// Constants used by WSHShell.Popup
var vbYesNoCancel = 3;
var vbQuestion = 32;
var vbYes = 6;
var vbNo = 7;
var vbExclamation = 48;
var vbInformation = 64;

var Install;
// Ask user if they really really want to do this
Install = WSHShell.Popup("This script will install or uninstall a fix for Sibelius or G7 with Windows XP Service Pack 2. Do not install unless you've installed XP Service Pack 2. Press Yes to install, No to un-install, or Cancel to exit without changes?", 60, ScriptName, vbYesNoCancel+vbQuestion);

// Any of the previous errors are fatal, from here we explicitly check for serious errors
// On Error Resume Next

if (Install == vbYes) {                 // User chose yes
    
	var ProgramName, Result, Pos, NewCommand, NewPrintCommand;   // Declare temporary variables used for storage
    
    var OldCommand = "";
	ProgramName = SibeliusProgramName;
	try {
	    OldCommand = WSHShell.RegRead (RegKey + OldVerb + CommandStr);	// Get the old command associated with the old verb
	} catch (e) {
        // failure to read this is handled below	
    }
    
	Pos = OldCommand.indexOf(ProgramName);	// Find our program name within the old command (0-based unlike vbscript)
	
	// If we don't have Sibelius handling Sibelius.Score then look for G7 instead.
	if (Pos < 0) {
	    ProgramName = G7ProgramName;
	    Pos = OldCommand.indexOf(ProgramName);	// Find our program name within the old command (0-based unlike vbscript)
    }

	if (Pos < 0) {							// Error - could not find our program name inside the file association - warn the user and give up
		WSHShell.Popup("Could not find the file association for Sibelius or G7 in your registry. Terminating without making changes.", 60, ScriptName, vbExclamation);
	} else {

		NewCommand = OldCommand.substr(0, Pos+(ProgramName.length));		// Trim off everything beyond our program name in the new verb

		// Calculate the new program string which is "<path><programname>""%1" without the space
		// XP SP2 will add the space when starting our program from Explorer
		NewCommand = NewCommand + "\"\"%1\"";						// this wierd string constant is actually ""%1"

		WSHShell.RegWrite(RegKey + NewVerb + CommandStr, NewCommand);	// Write the new command and new verb
		
		// Modify the Applications settings for g7.exe and Sibelius.exe if they exist to fix
		// users who've associated them with other file types
	    WriteApplicationKey(SibeliusProgramName, "");            // No space between program and arguments to fix SP2 bug
	    WriteApplicationKey(G7ProgramName, "");                  // No space between program and arguments to fix SP2 bug

        OldVerb = "";
        try {
		    OldVerb = WSHShell.RegRead (RegKey);				// get the previous default verb name
		} catch (e) {
		                                // ignore errors for this read - we can cope if it fails
	    }
		if (OldVerb != NewVerb && OldVerb.length > 0) {			// If it exists and it's not our new verb (e.g. they ran this script twice)
		    try {
			    WSHShell.RegWrite(RegKey + OldDefaultKey, OldVerb);	// write it to a backup key for undoing
			} catch (e) {
                                        // ignore errors for this write - we can cope if it fails
			}
		}
    
		WSHShell.RegWrite(RegKey, NewVerb);							// Write the new default verb 

		// Notify user that all is well
		WSHShell.Popup("Registry changes made successfully. If you want to undo these changes then re-run this script and choose 'No'.", 60, ScriptName, vbInformation);
	}
}

if (Install == vbNo) {                                  // User chose 'No' - undo previous changes

    OldVerb = "";
    try {
        // Read old default verb from backup key 
        OldVerb = WSHShell.RegRead (RegKey + OldDefaultKey); // Error here not fatal
    } catch (e) {
                                       // ignore errors for this write - we can cope if it fails
    }
    
    var deleteDefault = true;
    // first try to read the backed up old default verb
    if (OldVerb.length > 0) {
        try {
            // Writing default verb to be backed-up value
            WSHShell.RegWrite(RegKey, OldVerb);
            deleteDefault = false;
        } catch (e) {
        }
    }
    // If we can't read or set a backup then delete the default verb
    if (deleteDefault) {
        try {
            WSHShell.RegDelete(RegKey);
        } catch (e) {
        }
    }
    
    // Not too concerned if we can't delete this as it means the key probably isn't there. 
    try {
		WSHShell.RegDelete(RegKey + NewVerb + "\\command\\");   // Remove the entire key for the new verb
	} catch (e) {
	}
	// Not too concerned if we can't delete this as it means the key probably isn't there. 
	try {
		WSHShell.RegDelete(RegKey + NewVerb + "\\");           // Remove the entire key for the new verb
	} catch (e) {
	}
	
    WriteApplicationKey(SibeliusProgramName, " ");        // Add a space between program and arguments as before
    WriteApplicationKey(G7ProgramName, " ");              // Add a space between program and arguments as before
	
	WSHShell.Popup("Registry changes uninstalled successfully.", 60, ScriptName, vbInformation);
}

// Write an application key for ProgramName in HKCR\Applications
function WriteApplicationKey(ProgramName, Delimiter)
{
    var Pos, NewCommand;
    var OldCommand="";
    
    try {
	    OldCommand = WSHShell.RegRead (ApplicationKey + ProgramName + "\\shell\\open\\command\\");
	} catch (e) {
	    // if registry key not found then OldCommand will be empty and handled below
    }
	Pos = OldCommand.indexOf(ProgramName);		// Find our program name within the old command
    
	if (Pos >= 0) {
	    NewCommand = OldCommand.substr(0, Pos+(ProgramName.length));		// Trim off everything beyond our program name in the new verb
	    NewCommand = NewCommand + "\"" + Delimiter + "\"%1\"";		// this weird string is NewCommand"Delimiter"%1"
    
	    // Also write the old command in the Applications key used when users associated our program with another file type
	    WSHShell.RegWrite(ApplicationKey + ProgramName + "\\shell\\open\\command\\", NewCommand);
	}
}